/**************************************************************************************************|
|* Copyright 2020 Hycon Technology, Corp.                                                         *|
|* http://www.hycontek.com/                                                                       *|
|* Description:                                                                                   *|
|*                                                                                                *|
|**************************************************************************************************/

/**************************************************************************************************|
|*  Includes                                                                                      *|
|**************************************************************************************************/
#include <hy17p60bsfr.h>
#include <ADC.h>
#include <CLK.h>    
#include <PWR.h>     
/**************************************************************************************************|
|*  Static Function                                                                               *|
|**************************************************************************************************/

/**************************************************************************************************|
|*  Generic Peripheral Library                                                                    *|
|**************************************************************************************************/
/*-------------------------------------------------------------------------------------------------|
|  Function: ADC_Open                                                                              |
|                                                                                                  |
|  Parameters:                                                                                     |
|         ck        - [in] DADC_DHSCKDIV16   / DADC_DHSCKDIV8                                      |
|                          DADC_DHSCKDIV4                                                          |
|         inh       - [in] SAD1FP_OP1O / SAD1FP_VDDA / SAD1FP_REFO / SAD1FP_PB0 /                  |
|                          SAD1FP_PB2 / SAD1FP_PB4 / TS1P / TS1N                                   |
|         inl       - [in] SAD1FN_VSS / SAD1FN_AGND / SAD1FN_PB2 / SAD1FN_PB3 /                    | 
|                          SAD1FN_PB4 / TS2N / TS2P                                                |
|         vrh       - [in] SAD1RH_REFO / SAD1RH_PB4 / SAD1RH_VDDA / SAD1RH_AGND                    |
|         vrl       - [in] SAD1RL_AGND / SAD1RL_PB3 / SAD1RL_VSS                                   |
|         adgn      - [in] ADGN_1DIV2 / ADGN_1 / ADGN_2 / ADGN_4 / ADGN_8                          |
|         vrgn      - [in] VREGN_DIV1   / VREGN_DIV2                                               |
|         dcset     - [in] DCSET_P0     / DCSET_P1DIV8 / DCSET_P2DIV8 / DCSET_P3DIV8               |
|                          DCSET_P4DIV8 / DCSET_P5DIV8 / DCSET_P6DIV8 / DCSET_P7DIV8               |
|                          DCSET_N0     / DCSET_N1DIV8 / DCSET_N2DIV8 / DCSET_N3DIV8               |
|                          DCSET_N4DIV8 / DCSET_N5DIV8 / DCSET_N6DIV8 / DCSET_N7DIV8               |
|         osr       - [in] OSR_32    / OSR_64     / OSR_128 / OSR_256 / OSR_7680                   |
|                          OSR_15360 / OSR_30720  / OSR_61440                                      |
|                                                                                                  |
|  Returns:                                                                                        |
|         None                                                                                     |
|                                                                                                  |
|  Description:                                                                                    |
|         This function is used to open the ADC                                                    |
|-------------------------------------------------------------------------------------------------*/

void ADC_Open(unsigned char ck,    //Prescaler of SD18 peripheral operating frequency
              unsigned char inh,   //SI+ input signal selector
              unsigned char inl,   //SI- input signal selector
              unsigned char vrh,   //VR+ voltage signal selector
              unsigned char vrl,   //VR- voltage signal selector
              unsigned char adgn,  //AD amplification adjuster
              unsigned char vrgn,  //VR amplification adjuster
              unsigned char dcset, //SI+/- bias adjuster
              unsigned char osr)   //SD18 over-sampling rate divider
              
{
	// ADC CLK setting //
    ADC_CLK_DivSelect(ck);             //ADC_CK should keep on 1MHz
    // ADC Init //
    ADC_INPBufferEnable();             //Enable INP Buffer
	ADC_INNBufferEnable();             //Enable INN Buffer
	ADC_VRHBufferEnable();             //Enable VRH Buffer
	ADC_VRLBufferEnable();             //Enable VRL Buffer
	ADC_VRINSelect(vrh,vrl);           // Config ADC Ref. Voltage Input Channel
    ADC_ADGNSelect(adgn);              // Config AD Gain
    ADC_VREGNSelect(vrgn);             // Config Ref. Gain
    ADC_DCSETSelect(dcset);            // Config DC offset
    ADC_OSRSelect(osr);                // Config Over sampling ratio
    ADC_SFT1Select(SFT1_DISABLE);      //SAD1F shorted to SAD1I	
    if((inh == TS1N)|(inl == TS2P))       //select INP=TS1N/INN=TS2P
    {
        ADC_VINSelect(SAD1I_TS1N_TS2P);
    }
    else if((inh == TS1P)|(inl == TS2N))  //select INP=TS1P/INN=TS2N
    {
        ADC_VINSelect(SAD1I_TS1P_TS2N);   
    }
    else                                  //select INP=AD1FP/INN=AD1FN
    {
        ADC_VINSelect(SAD1I_AD1FP_AD1FN); 
        ADC_AD1FSelect(inh,inl);          //Select AD1FP and AD1FN Input Channel
    }
    
    ADIF_ClearFlag();
    ADIE_Enable();
    ADC_Enable();
    ADC_CMFREnable();

}

/*-------------------------------------------------------------------------------------------------|
|  Function: ADC_GetData                                                                           |
|                                                                                                  |
|  Parameters:                                                                                     |
|         None                                                                                     |
|                                                                                                  |
|  Returns:                                                                                        |
|         ADCData   - [in] Read the 24-bit data for ADCR                                           |
|                                                                                                  |
|  Description:                                                                                    |
|         This function is used to Read the Data for ADC                                           |
|-------------------------------------------------------------------------------------------------*/
long ADC_GetData(void)
{
    long ADCData;
    ((char *)(&ADCData))[0] = AD1DATAL;
    ((char *)(&ADCData))[1] = AD1DATAH;
    ((char *)(&ADCData))[2] = AD1DATAU;

    if (ADCData&0x800000L)
        ((char *)(&ADCData))[3] = 0xFF;
    else
        ((char *)(&ADCData))[3] = 0x00;
    return ADCData;
}
